"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));

var _react = _interopRequireDefault(require("react"));

var _reactRedux = require("react-redux");

var _actions = require("redux-query/src/actions");

var _queryKey = require("redux-query/src/lib/query-key");

var _useConstCallback = _interopRequireDefault(require("../hooks/use-const-callback"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const normalizeToArray = maybe => {
  return (Array.isArray(maybe) ? maybe : [maybe]).filter(Boolean);
};

const difference = (a, b) => {
  const bSet = new Set(b);
  return a.filter(x => !bSet.has(x));
};

const getDiff = (prevQueryConfigs, queryConfigs) => {
  const prevQueryKeys = prevQueryConfigs.map(_queryKey.getQueryKey);
  const queryKeys = queryConfigs.map(_queryKey.getQueryKey);
  const queryConfigByQueryKey = queryKeys.reduce((accum, queryKey, i) => {
    accum.set(queryKey, queryConfigs[i]);
    return accum;
  }, new Map());
  const cancelKeys = difference(prevQueryKeys, queryKeys);
  const requestKeys = difference(queryKeys, prevQueryKeys);
  const requestQueryConfigs = requestKeys.map(queryKey => queryConfigByQueryKey.get(queryKey));
  return {
    cancelKeys,
    requestQueryConfigs
  };
};

const useMemoizedQueryConfigs = (mapPropsToConfigs, props, callback) => {
  const queryConfigs = normalizeToArray(mapPropsToConfigs(props)).map(queryConfig => _objectSpread({}, queryConfig, {
    unstable_preDispatchCallback: () => callback((0, _queryKey.getQueryKey)(queryConfig))
  }));

  const [memoizedQueryConfigs, setMemoizedQueryConfigs] = _react.default.useState(queryConfigs);

  const previousQueryKeys = _react.default.useRef(queryConfigs.map(_queryKey.getQueryKey));

  _react.default.useEffect(() => {
    const queryKeys = queryConfigs.map(_queryKey.getQueryKey);

    if (queryKeys.length !== previousQueryKeys.current.length || queryKeys.some((queryKey, i) => previousQueryKeys.current[i] !== queryKey)) {
      previousQueryKeys.current = queryKeys;
      setMemoizedQueryConfigs(queryConfigs);
    }
  }, [queryConfigs]);

  return memoizedQueryConfigs;
};

const useMultiRequest = (mapPropsToConfigs, props) => {
  const reduxDispatch = (0, _reactRedux.useDispatch)();

  const previousQueryConfigs = _react.default.useRef([]);

  const pendingRequests = _react.default.useRef(new Set());

  const dispatchRequestToRedux = (0, _useConstCallback.default)(queryConfig => {
    const promise = reduxDispatch((0, _actions.requestAsync)(queryConfig));

    if (promise) {
      pendingRequests.current.add((0, _queryKey.getQueryKey)(queryConfig));
    }
  });
  const dispatchCancelToRedux = (0, _useConstCallback.default)(queryConfig => {
    reduxDispatch((0, _actions.cancelQuery)(queryConfig));
    pendingRequests.current.delete((0, _queryKey.getQueryKey)(queryConfig));
  });
  const queryConfigs = useMemoizedQueryConfigs(mapPropsToConfigs, props, queryKey => {
    pendingRequests.current.delete(queryKey);
  });

  _react.default.useEffect(() => {
    const {
      cancelKeys,
      requestQueryConfigs
    } = getDiff(previousQueryConfigs.current, queryConfigs);
    requestQueryConfigs.forEach(dispatchRequestToRedux);
    cancelKeys.forEach(dispatchCancelToRedux);
    previousQueryConfigs.current = queryConfigs;
    return () => {
      [...pendingRequests.current].forEach(dispatchCancelToRedux);
    };
  }, [dispatchCancelToRedux, dispatchRequestToRedux, queryConfigs]);

  const forceRequest = _react.default.useCallback(() => {
    queryConfigs.forEach(requestReduxAction => {
      dispatchRequestToRedux(_objectSpread({}, requestReduxAction, {
        force: true
      }));
    });
  }, [dispatchRequestToRedux, queryConfigs]);

  return forceRequest;
};

const connectRequest = (mapPropsToConfigs, options = {}) => WrappedComponent => {
  const {
    pure = true,
    forwardRef = false
  } = options;

  const ConnectRequestFunction = props => {
    const forceRequest = useMultiRequest(mapPropsToConfigs, props);
    return _react.default.createElement(WrappedComponent, _extends({}, props, {
      forceRequest: forceRequest
    }));
  };

  const ConnectRequest = pure ? _react.default.memo(ConnectRequestFunction) : ConnectRequestFunction;
  const wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
  const displayName = `ConnectRequest(${wrappedComponentName})`;
  ConnectRequest.WrappedComponent = WrappedComponent;
  ConnectRequest.displayName = displayName;

  if (forwardRef) {
    const forwarded = _react.default.forwardRef((props, ref) => _react.default.createElement(ConnectRequest, _extends({}, props, {
      forwardedRef: ref
    })));

    forwarded.displayName = displayName;
    forwarded.WrappedComponent = WrappedComponent;
    return (0, _hoistNonReactStatics.default)(forwarded, WrappedComponent);
  }

  return (0, _hoistNonReactStatics.default)(ConnectRequest, WrappedComponent);
};

var _default = connectRequest;
exports.default = _default;